/*****************************************************************************
 * Copyright (c) 2014 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  CEA LIST - Initial API and implementation
 *  Celine JANSSENS (ALL4TEC) celine.janssens@all4tec.net - Add #getStereotypeApplication()
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.utils;

import org.eclipse.papyrusrt.umlrt.core.types.ElementTypeUtils;
import org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes;
import org.eclipse.papyrusrt.umlrt.profile.UMLRealTime.CapsulePart;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.util.UMLUtil;

/**
 * Utility class for capsule parts
 */
public class CapsulePartUtils {

	/**
	 * Returns <code>true</code> if the specified element is a CapsulePart (property typed by a capsule, and stereotyped by <<CapsulePart>>)
	 * 
	 * @param property
	 *            the property to check
	 * @return <code>true</code> if the specified element is a CapsulePart (property typed by a capsule, and stereotyped by <<CapsulePart>>)
	 */
	public static boolean isCapsulePart(final Property property) {
		return ElementTypeUtils.matches(property, IUMLRTElementTypes.CAPSULE_PART_ID);
	}

	/**
	 * Get the Stereotype Application of the Capsule Part
	 * 
	 * @param property
	 *            The Capsule Part as a Property
	 * @return the Capsule Part stereotype application from the UML RT profile application.
	 */
	public static CapsulePart getStereotypeApplication(final Property property) {
		return UMLUtil.getStereotypeApplication(property, CapsulePart.class);

	}

}
