/*****************************************************************************
 * Copyright (c) 2016 Christian W. Damus and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Christian W. Damus - Initial API and implementation
 *  
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.utils;

import static org.eclipse.papyrusrt.umlrt.core.types.ElementTypeUtils.getConfigureAsCommand;

import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.papyrusrt.umlrt.core.types.ElementTypeUtils;
import org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes;
import org.eclipse.papyrusrt.umlrt.core.types.UMLRTElementTypesEnumerator;
import org.eclipse.uml2.uml.State;

/**
 * Utilities for inspecting and working with states in an UML-RT model.
 */
public class StateUtils {

	/**
	 * Queries whether a {@code state} is stereotyped as an UML-RT state.
	 * 
	 * @param state
	 *            a state
	 * 
	 * @return whether it is an <tt>RTState</tt>
	 */
	public static boolean isRTState(State state) {
		return ElementTypeUtils.matches(state, IUMLRTElementTypes.RT_STATE_ID);
	}

	/**
	 * Obtains a command that ensures that a {@code state} is stereotyped as an
	 * UML-RT state.
	 * 
	 * @param state
	 *            a state
	 * 
	 * @return the command
	 */
	public static ICommand getConfigureAsRTRegionCommand(State state) {
		return getConfigureAsCommand(UMLRTElementTypesEnumerator.RT_STATE, state);
	}
}
